#!/bin/sh

DRIVER=3w-sas

echo "Driver Installation"

MACHINE=`uname -m`
KERNEL=`uname -r`
export PATH=/bin:/sbin:/usr/bin:/usr/sbin
RELEASE="UNKNOWN"
INITRD=initrd
( echo $KERNEL | grep "fc12" -s -q ) && { INITRD=initramfs; RELEASE=fc12; }

mkdir /tmp/modules
(cd /tmp/modules; gzip -dc /disk/modules.cgz | cpio -idum) 1>/dev/null 2>&1 || { echo "Fail to extract drivers!"; exit 1; }

for ver in /lib/modules/*; do
	kver=`basename $ver`
	mod=`find /lib/modules/$kver/kernel/fs -type f -name *.ko | head -1`
	vermagic=`modinfo -F vermagic $mod 2> /dev/null`
	[ "$vermagic" = "" ] && vermagic=`strings $mod | grep vermagic`
	( echo "$vermagic" | grep " 586 " -s -q ) && MACHINE=i586
	( echo "$vermagic" | grep " 686 " -s -q ) && MACHINE=i686
	( echo "$vermagic" | grep " x86_64 " -s -q ) && MACHINE=x86_64

	if [ -d /lib/modules/${kver} -a -d /tmp/modules/${kver} ]; then
		echo "Updating ${kver}..."
		mv /boot/${INITRD}-${kver}.img /boot/${INITRD}-${kver}.img.save && 
		cp /tmp/modules/${kver}/${DRIVER}.ko /lib/modules/${kver}/kernel/drivers/scsi/ &&
		/sbin/depmod -a ${kver}

		if [ $? = 1 ]; then
			echo "Postinstall 1 failed! Please reinstall your GNU/Linux system."
			exit
		fi

		( grep -s -q scsi_mod /lib/modules/${kver}/modules.dep ) && PRESCSI="--preload scsi_mod"
		( grep -s -q sd_mod /lib/modules/${kver}/modules.dep ) && PRESD="--preload sd_mod"
		( grep -s -q scsi_wait_scan /lib/modules/${kver}/modules.dep ) && WITHWAIT="--with=scsi_wait_scan"
		   if [ $RELEASE = "fc12" ]; then
                     /sbin/dracut --add-drivers ${DRIVER} -f /boot/${INITRD}-${kver}.img ${kver}
                     else
                     /sbin/mkinitrd ${PRESCSI} ${PRESD} --with=${DRIVER} ${WITHWAIT} /boot/${INITRD}-${kver}.img ${kver}
                    fi


		if [ $? = 1 ]; then
			echo "Postinstall 2 failed! Please reinstall your GNU/Linux system."
			exit
		fi
	fi
done

rm -fr /tmp/modules

echo "Driver installation step 2 completed."
